# Maintainers' guide

This section of documentation is for Zeno maintainers.

## Translating Zeno editor UI

First update the translation file `zh.ts` using the following command:
```bash
lupdate -recursive ui/zenoedit/ -ts ui/zenoedit/res/languages/zh.ts
```

Then open that `zh.ts` file with Qt Linguist:
```bash
linguist ui/zenoedit/res/languages/zh.ts
```

You can now edit the translations.
Now select an entry, type the correct Chinese translation.
Then press `Ctrl-Enter` to submit your translation.

After finish, press the `Release` in Qt Linguist, you will get `ui/zenoedit/res/languages/zh.qm`.
Now rebuild Zeno, and you will see your updated translations in UI.

This process can be ran for multiple times, `lupdate` won't override the old translations, no worry.

> See also `misc/trans.sh`.

## Deploying Zeno (CI/CD)

The deployment is automated via GitHub CI, see `.github/workflows/cmake.yml`.

To trigger it, simply push a commit message containing `[release]`, for example:

```bash
git commit -m "[release] some description"
git push
```

Push it, then GitHub CI will do the depolyment automatically for you.

It will create a release with tag, for example, `v2022.4.19` (today's date).

# CMake tricks & tips

## Accelerate compile process

You may install `ccache` to compile faster on Linux.

```bash
sudo apt-get install -y ccache
```

```bash
sudo pacman -S ccache
```

You may install `ninja` (a faster build system than `make`).

```bash
sudo apt-get install -y ninja
```

```bash
sudo pacman -S ninja
```

```bash
pip install ninja
```

Then use `-G Ninja` parameter in cmake configuration step:

```bash
cmake -B build -G Ninja -DCMAKE_BUILD_TYPE=Release
cmake --build build --parallel
```

## Using pre-installed libraries

By default, Zeno uses bundled OpenVDB (as submodule).

If you wish to use the system-wide pre-installed OpenVDB (for faster compile speed), use:

```bash
cmake -B build -DZENO_WITH_zenvdb:BOOL=ON -DZENO_SYSTEM_OPENVDB:BOOL=ON
```

> NOTE: the pre-installed OpenVDB might cause error in @littlemine's CUDA extension, use `-DZENO_SYSTEM_OPENVDB:BOOL=OFF` if you got error.

The same applies to Alembic (required for `.abc` format support):

```bash
cmake -B build -DZENO_WITH_Alembic:BOOL=ON -DZENO_SYSTEM_ALEMBIC:BOOL=ON
```

# Alternative installation

## Install Zeno globally (not recommended)

For the Linux traditional `make install` style installation, please specify this argument:

```bash
cmake -DZENO_INSTALL_TARGET:BOOL=ON -DCMAKE_INSTALL_PREFIX=/usr/local
```

> Change the `/usr/local` to your preferred install path, for example `/opt/zeno-2022.4.19`.

Next, build and install via CMake:

```bash
cmake --build build
sudo cmake --build build --target install
```

This will install Zeno globally in your system.

## Install via AUR (experimental)

Arch Linux users may install Zeno from [AUR](https://aur.archlinux.org):
```bash
yay -S zeno
```
The package is maintained by @archibate.

